/*
 * Serial port driver for MCD direct-c call
 * rs232c.c version 1.00
 * Copyright (C) 1997 by Y.Akagawa
 *
 *  int rs_init( int, int, int, int, int );
 *  int getaux( void );
 *  int putaux( int );
 *  int checkaux( void );
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>

#include "local.h"
#include "ioctl.h"
#include "mcdif.h"
#include "mcdfunc.h"
#include "rs232c.h"

static int auxfd;
static int _stop;

static void (far *mcdfunc)(void);

/* DIRECT C CALL (mcd ver 0.8) */
void (far *mcd_getid)(get_id_t far *buf);
void (far *mcd_linepara)(int f, line_para_t far *param);
word (far *mcd_linestat)(int f, ...);
void (far *mcd_bufstat)(int f, ...);
word (far *mcd_blockread)(char far *buf, word size);
word (far *mcd_blockwrite)(char far *buf, word size);
int (far *mcd_breaksig)(int f);
void (far *mcd_buffer)(int f, ...);
int (far *mcd_getc)(void);
int (far *mcd_putc)(int c);
word (far *mcd_getsbufchars)(void);
word (far *mcd_getsbuffree)(void);
word (far *mcd_getrbufchars)(void);
word (far *mcd_getrbuffree)(void);
int (far *mcd_ndread)(void);
dword (far *mcd_speed)(int f, ...);
void (far *mcd_flwsize)(int f, ...);
void (far *mcd_devopen)(int f, ...);

int ioread(int handle, int buflen, char far *bufptr)
{
	unsigned int REG_AX;
	_asm{
		push	ds
		lds	dx, bufptr
		mov	ax, 4402h
		mov	bx, handle
		mov	cx, buflen
		int	21h
		pop	ds
		jnc	fin
		mov	ax, 0
fin:
		mov REG_AX, ax
	}
	return REG_AX;
}
static int mcd_read(int func, int size, void *pac)
{
	static iopac_t iopac;
	int r;

	iopac.func = func;
	memcpy(&iopac.body, pac, size);
	r = ioread(auxfd, size + sizeof (iopac.func), (char *)&iopac);
	memcpy(pac, &iopac.body, size);
	return r;
}

int mcd_c_entry(void)
{
	get_c_entry_t entry;
	dword cseg;
#define SETENT(r, a, n)	(r (far *)a)(cseg | entry.offs[n])

	if (!mcd_read(F_GET_C_ENTRY, L_GET_C_ENTRY, (char *) &entry)) {
		return 0;
	}
	cseg = ((dword)entry.cseg) << 16;
	mcd_getid = SETENT(void, (get_id_t far *), 0);
	mcd_linepara = SETENT(void, (int, line_para_t far *), 1);
	mcd_linestat = SETENT(word, (int, ...), 2);
	mcd_bufstat = SETENT(void, (int, ...), 3);
	mcd_blockread = SETENT(word, (char far *, word), 4);
	mcd_blockwrite = SETENT(word, (char far *, word), 5);
	mcd_breaksig = SETENT(int, (int), 6);
	mcd_buffer = SETENT(void, (int, ...), 7);
	mcd_getc = SETENT(int, (void), 8);
	mcd_putc = SETENT(int, (int), 9);
	mcd_getsbufchars = SETENT(word, (void), 10);
	mcd_getsbuffree = SETENT(word, (void), 11);
	mcd_getrbufchars = SETENT(word, (void), 12);
	mcd_getrbuffree = SETENT(word, (void), 13);
	mcd_ndread = SETENT(int, (void), 14);
	mcd_speed = SETENT(dword, (int, ...), 15);
	mcd_flwsize = SETENT(void, (int, ...), 16);
	mcd_devopen = SETENT(void, (int, ...), 17);
	return 1;
#undef SETENT
}

int mcd_check(int *maj, int *min)
{
	static char id[10];
	static get_id_t drv_id = { 0, 0, (byte far *)id };
	get_entry_t get_entry;

	if (mcd_read(F_GET_ID, L_GET_ID, &drv_id) == 0)
	{
		printf("cannot get driver-id\n");
		return 0;
	}
	if (strcmp(id, "MCD"))
	{
		printf("driver-id mismatch: %s\n", id);
		return 0;
	}
	if (drv_id.maj < *maj || (drv_id.maj == *maj &&  drv_id.min < *min))
	{
		printf("driver is old version: %d.%d\n", drv_id.maj, drv_id.min);
		return 0;
	}
	*maj = drv_id.maj;
	*min = drv_id.min;
	if (mcd_read(F_GET_ENTRY, L_GET_ENTRY, &get_entry) == 0)
	{
		printf("cannot get driver entry\n");
		return 0;
	}
	mcdfunc = get_entry.func;
	if (mcd_c_entry() == 0)
	{
		printf("Cannot get MCD C ENTRY.\n");
		return 0;
	}
	return 1;
}

int mcd_open(char *name, int *maj_ver, int *min_ver)
{
	if ((auxfd = open(name, O_BINARY | O_RDWR)) == -1) {
		printf("can't open <%s\n>\n", name);
		return 0;
	}
	if (mcd_check(maj_ver, min_ver) == 0)
		return 0;
	return 1;
}

void mcd_close(void)
{
	close(auxfd);
}


int rs_init(int bps, int parity, int stop, int length, int flow)
{
	int maj_ver = MAJOR_VER;
	int min_ver = MINOR_VER;
	line_para_t mcdline;
	
	if(mcd_open("AUX", &maj_ver, &min_ver) == 0)
		return 1;
	
	mcdline.baud   = bps;
	mcdline.parity = parity;
	mcdline.stop   = stop;
	mcdline.length = length;

	/* flow control */
	switch(flow)
	{
		case FC_RTSCTS :
			mcdline.flow = 0x0002;
			break;
		case FC_DTRDSR :
			mcdline.flow = 0x0004;
			break;
		case FC_RXONXOFF :
			mcdline.flow = 0x8001;
			break;
		case FC_SXONXOFF :
			mcdline.flow = 0x8100;
			break;
		default :
			mcdline.flow = 0x0000;
			break;
	}

	mcd_linepara(1, &mcdline);
	atexit(mcd_close);
	_stop = 0;
	return 0;
}

static unsigned char sencekey()
{
	unsigned char reg_al;
	_asm{
		mov		ah, 006h
		mov		dl, 0ffh
		int		21h
		mov		reg_al, al
	}
	return reg_al;
}

int checkaux()
{
	int c;
	if(sencekey() == 0x1b)
	{
		_stop = 1;
		return -2;
	}
	c = mcd_ndread();
	if(c == -1)
		return -1;
	else
		return 0;
}

int getaux()
{
	if(sencekey() == 0x1b)
	{
		_stop = 1;
		return -2;
	}
	return mcd_getc();
}

int putaux(int c)
{
	while(1)
	{
		if(sencekey() == 0x1b)
		{
			_stop = 1;
			return -2;
		}
		if(mcd_getsbufchars() == 0)
			break;
	}
	mcd_putc(c);
	return 0;
}
